module Alucard
  class Card
    RANKS = 'A23456789TJQK'
    SUITS = 'SHCD'
    RANK_NAMES = %w[Ace Two Three Four Five Six Seven Eight Nine Ten Jack Queen King]
    SUIT_NAMES = %w[Spades Hearts Clubs Diamonds]

    attr_reader :rank, :suit

    def initialize spec
      case spec
      when Fixnum
        @rank, @suit = spec % 13, spec / 13
      when /([#{RANKS}])([#{SUITS}])/
        @rank, @suit = RANKS.index($1), SUITS.index($2)
      when /(\w+) of (\w+)/
        r, s = $1.capitalize, $2.capitalize

        raise ArgumentError, "Not a rank name: '#{r}'" unless RANK_NAMES.include? r
        raise ArgumentError, "Not a suit name: '#{s}'" unless SUIT_NAMES.include? s

        @rank, @suit = RANK_NAMES.index(r), SUIT_NAMES.index(s)
      else
        raise ArgumentError, "Bad card spec: '#{spec}'"
      end
    end

    def to_s
      RANKS[@rank] + SUITS[@suit]
    end
  end
end
